/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "interface.h"
#include "support.h"
#include "math.h"
#include "main.h"

float INITIAL_XMAX=5.0, INITIAL_XMIN=-5.0, INITIAL_YMAX=4.0, INITIAL_YMIN=-4.0;
float MOVE_FRACTION_X = 0.05;
float MOVE_FRACTION_Y = 0.05;
float SCREEN_FRACTION_X = 0.1;
float SCREEN_FRACTION_Y = 0.1;

guchar rgbbuf[WIDTH * HEIGHT * 3];
GtkWidget *mainWindow;
GtkWidget *darea;

float xmax_queue[POINT_BUF_LENGTH];
float xmin_queue[POINT_BUF_LENGTH];
float ymax_queue[POINT_BUF_LENGTH];
float ymin_queue[POINT_BUF_LENGTH];
int points;

void init(void);
void plot_picture(void);
void paint_picture(void);
void update(void);
int count(int, int);
int iscomplex(int);


gboolean expose_event(GtkWidget *widget, GdkEvent *event, gpointer data);
gboolean tick_event(gpointer data);

gint flameCount=0;

float xmin, xmax, ymin, ymax, t_xmin, t_xmax, t_ymin, t_ymax;

int
main (int argc, char *argv[])
{
  srandom(time(NULL));
  gnome_init ("gnofractroam", VERSION, argc, argv);
  gdk_rgb_init();

  mainWindow = create_mainWindow ();
  darea = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA (darea), WIDTH, HEIGHT);
  gtk_container_add(GTK_CONTAINER(mainWindow),darea);
  gtk_signal_connect(GTK_OBJECT(darea), "expose-event", GTK_SIGNAL_FUNC(expose_event),NULL);
  gtk_widget_show_all (mainWindow);
  
  init();

  plot_picture();
  gtk_timeout_add(UPDATE,tick_event,NULL);

  gtk_main ();
  return 0;
}

int count(int x, int y) {
  float z=0, zi=0, zprev, c, ci;
  int n;
  c = xmin+x*(xmax-xmin)/WIDTH;
  ci = ymin+y*(ymax-ymin)/HEIGHT;
  for (n=0;n<256 && z < 2;n++) {
    zprev=z;
    z = z*z - zi*zi + c;
    zi= 2 * zprev * zi + ci;
  }
  return n;
}

void init() {
  xmin = INITIAL_XMIN; xmax = INITIAL_XMAX;
  ymin = INITIAL_YMIN; ymax = INITIAL_YMAX;
  t_xmax=xmax;
  t_xmin=xmin;
  t_ymax=ymax;
  t_ymin=ymin;
}

void plot_picture() {
  gint x, y;
  guchar *pos;
  int temp;
  pos = rgbbuf;
  
  for (y = 0; y < HEIGHT; y++) {
    for (x = 0; x < WIDTH; x++) {
      temp=5*count(x,y);
      *pos++ = temp;
      *pos++ = temp;
      *pos++ = temp;
    }
  }
}

int iscomplex(int point) {
  int x = point % WIDTH;
  int y = point / WIDTH;
  int up,left,right,down,upleft,upright,downleft,downright,center;
  center = rgbbuf[(x+(WIDTH*y))*3];
  if (center > THRESH) {
    up = rgbbuf[(x+WIDTH*(y+1))*3];
    down = rgbbuf[(x+WIDTH*(y-1))*3];
    left = rgbbuf[(x-1+WIDTH*y)*3];
    right = rgbbuf[(x+1+WIDTH*y)*3];
    upleft = rgbbuf[(x-1+WIDTH*(y+1))*3];
    downleft = rgbbuf[(x-1+WIDTH*(y-1))*3];
    upright = rgbbuf[(x+1+WIDTH*(y+1))*3];
    downright = rgbbuf[(x+1+WIDTH*(y-1))*3];
    return (center != up && center != down && center != left && center != right && center != upleft && center != downleft && center != upright && center != downright && upleft != upright && upleft != downleft && upright != downright);
  }
  return 0;
}

void paint_picture(){
  gdk_draw_rgb_image (darea->window, darea->style->fg_gc[GTK_STATE_NORMAL], 0, 0, WIDTH,HEIGHT, GDK_RGB_DITHER_MAX, rgbbuf, WIDTH * 3);
}

void update() {
  float dist;
  float xbound, ybound, xpixel, ypixel;
  static float lastxpixel, lastypixel;
  float point1x=0, point1y=0, point2x=0, point2y=0;
  int x, y, point, mycount=0;
  xpixel = (xmax-xmin)/WIDTH;
  ypixel = (ymax-ymin)/HEIGHT;
  xbound = fabs(xpixel) * BOUND;
  ybound = fabs(ypixel) * BOUND;

printf("  x: %e to %e; y: %e to %e\n",xmin,xmax,ymin,ymax);
printf("->x: %e to %e; y: %e to %e\n",t_xmin,t_xmax,t_ymin,t_ymax);
printf("xpixel: %e, ypixel: %e\n\n",xpixel,ypixel);

  //if we're within BOUND pixels for xmax, xmin, ymax, ymin -- pick a point
  if ((lastxpixel == xpixel || lastypixel == ypixel) || (fabs(xmax-t_xmax) < xbound && fabs(xmin-t_xmin) < xbound && fabs(ymax-t_ymax) < ybound && fabs(ymin-t_ymin) < ybound)) {
    //if there is anything on the queue, 50/50 to pick off it
    if (!(xmax_queue[points-1] == 0 && xmin_queue[points-1] == 0) && ((lastxpixel == xpixel || lastypixel == ypixel) || (points > 0 && ((double)random()/RAND_MAX) > .5))){
      points--;
      t_xmax = xmax_queue[points];
      t_xmin = xmin_queue[points];
      t_ymax = ymax_queue[points];
      t_ymin = ymin_queue[points];
    } else {
      //take random samples until two points are above threshold
      while (point2y == 0 && mycount < MAXCOUNT) {
        point = (int) fabs(((double)random()/RAND_MAX) * WIDTH * HEIGHT);
        mycount++;
        if (iscomplex(point)) {
          x = point % WIDTH;
          y = point / WIDTH;
          if (point1y == 0) {
            point1x = xmin + x * xpixel;
            point1y = ymin + y * ypixel;
          } else {
            point2x = xmin + x * xpixel;
            point2y = ymin + y * ypixel;
          }
        }
      }
      //set target bounds to bound that point as center
      //using SCREEN_FRACTION_X/Y
      if (mycount < MAXCOUNT) {
        t_xmax = point1x + (xmax-xmin)*(SCREEN_FRACTION_X/2);
        t_xmin = point1x - (xmax-xmin)*(SCREEN_FRACTION_X/2);
        t_ymax = point1y + (ymax-ymin)*(SCREEN_FRACTION_Y/2);
        t_ymin = point1y - (ymax-ymin)*(SCREEN_FRACTION_Y/2);
        xmax_queue[points] = point2x + (xmax-xmin)*(SCREEN_FRACTION_X/2);
        xmin_queue[points] = point2x - (xmax-xmin)*(SCREEN_FRACTION_X/2);
        ymax_queue[points] = point2y + (ymax-ymin)*(SCREEN_FRACTION_Y/2);
        ymin_queue[points] = point2y - (ymax-ymin)*(SCREEN_FRACTION_Y/2);
        points++;
      } else {
        //we're not able to find anything interesting, let's start over...
        points=0;
        t_xmax = INITIAL_XMAX;
        t_xmin = INITIAL_XMIN;
        t_ymax = INITIAL_YMAX;
        t_ymin = INITIAL_YMIN;
      }
//printf("COUNT: %d\n",mycount);
    }
  }
  //move MOVE_FRACTION_X/Y
  dist = (xmax-xmin)/WIDTH;
  xmax += (t_xmax - xmax)* MOVE_FRACTION_X;
  xmin += (t_xmin - xmin)* MOVE_FRACTION_X;
  dist = (ymax-ymin)/HEIGHT;
  ymax += (t_ymax - ymax)* MOVE_FRACTION_Y;
  ymin += (t_ymin - ymin)* MOVE_FRACTION_Y;
  lastxpixel=xpixel;
  lastypixel=ypixel;
  plot_picture();
}

gboolean expose_event(GtkWidget *widget, GdkEvent *event, gpointer data) {
  paint_picture();
  return TRUE;
}

gboolean tick_event(gpointer data) {
  update();
  paint_picture();
  return TRUE;
}
